import re

STC_METHOD_FUNC_MATCHER1 = re.compile("__x___M_\d+_([A-Za-z_]\w*)_(_.*)")
STC_METHOD_FUNC_MATCHER2 = re.compile("__M_\d+_([A-Za-z_]\w*)_(_.*)")
STC_BLOCK_FUNC_MATCHER  = re.compile("([A-Za-z_]\w*)__B_\d+")

STC_DEMANGLE_TABLE_1 = {
    'pl'  : '+', 'mi'  : '-',
    'mu'  : '*', 'di'  : '/',
    'co'  : ',', 'at'  : '@', 
    'le'  : '<', 'gr'  : '>',
    'eq'  : '=', 'ne'  : '~',
    'pi'  : '|', 'mo'  : '\\',
    'am'  : '&'
}

def demangle(fname):
    """
    Given a C function name mangled by STX, returns tupe (classname, selector). 
    For block functions, return tuple (classname, None) as stc does not mangle
    method name into block C function name. If given fname is not mangled 
    method name nor block function name, return tupe (None, None).
    """

    def selector(cselector):
        """
        Demangle given selector into it's Smalltalk form. 
        See http://live.exept.de/doc/online/english/programming/primitive.html#ACCESS_SYMBOLS
        """


        sselector = cselector[1:]
        return re.sub("_", ":", sselector)


    match = STC_METHOD_FUNC_MATCHER1.match(fname)
    if match != None:
        return ( match.group(1) , selector(match.group(2)) )
    match = STC_METHOD_FUNC_MATCHER2.match(fname)
    if match != None:
        return ( match.group(1) , selector(match.group(2)) )
 
    match = STC_BLOCK_FUNC_MATCHER.match(fname)
    if match != None:
        return ( match.group(1) , None )
    return ( None, None )


